@@ -55,7 +55,7 @@ class GraphicsStateMixin:
                 nom_lift=0.2,
                 denom_lift=0.0,
                 text_shaping=None,
-            ),
+            )
         ]
         super().__init__(*args, **kwargs)
 
@@ -76,6 +76,9 @@ class GraphicsStateMixin:
         gs["text_shaping"] = copy(gs["text_shaping"])
         return gs
 
+    def _is_current_graphics_state_nested(self):
+        return len(self.__statestack) > 1
+
     @property
     def draw_color(self):
         return self.__statestack[-1]["draw_color"]
@@ -338,8 +341,7 @@ class GraphicsStateMixin:
 
     @text_shaping.setter
     def text_shaping(self, v):
-        if v:
-            self.__statestack[-1]["text_shaping"] = v
+        self.__statestack[-1]["text_shaping"] = v
 
     def font_face(self):
         """
