@@ -1,4 +1,11 @@
-"Document signature generation"
+"""
+Module dedicated to document signature generation.
+
+The contents of this module are internal to fpdf2, and not part of the public API.
+They may change at any time without prior warning or any deprecation period,
+in non-backward-compatible ways.
+"""
+
 import hashlib
 from datetime import timezone
 from unittest.mock import patch
@@ -24,8 +31,12 @@ class Signature:
         self.byte_range = _SIGNATURE_BYTERANGE_PLACEHOLDER
         self.contents = "<" + _SIGNATURE_CONTENTS_PLACEHOLDER + ">"
 
-    def serialize(self):
-        obj_dict = build_obj_dict({key: getattr(self, key) for key in dir(self)})
+    def serialize(self, _security_handler=None, _obj_id=None):
+        obj_dict = build_obj_dict(
+            {key: getattr(self, key) for key in dir(self)},
+            _security_handler=_security_handler,
+            _obj_id=_obj_id,
+        )
         return pdf_dict(obj_dict)
 
 
