@@ -4,8 +4,9 @@ __author__ = "Mariano Reingart <reingart@gmail.com>"
 __copyright__ = "Copyright (C) 2010 Mariano Reingart"
 __license__ = "LGPL 3.0"
 
-import csv, locale, warnings
+import os, csv, json, locale, warnings
 
+from .deprecation import get_stack_level
 from .errors import FPDFException
 from .fpdf import FPDF
 
@@ -31,13 +32,9 @@ class FlexTemplate:
 
     def __init__(self, pdf, elements=None):
         """
-        Arguments:
+        Arguments: pdf (fpdf.FPDF() instance): All content will be added to this object.
 
-            pdf (fpdf.FPDF() instance):
-                All content will be added to this object.
-
-            elements (list of dicts):
-                A template definition in a list of dicts.
+            elements (list of dicts): A template definition in a list of dicts.
                 If you omit this, then you need to call either load_elements()
                 or parse_csv() before doing anything else.
         """
@@ -65,8 +62,7 @@ class FlexTemplate:
 
         Arguments:
 
-            elements (list of dicts):
-                A template definition in a list of dicts
+            elements (list of dicts): A template definition in a list of dicts
         """
         key_config = {
             # key: type
@@ -88,6 +84,7 @@ class FlexTemplate:
             "priority": int,
             "multiline": (bool, type(None)),
             "rotate": (int, float),
+            "wrapmode": (str, type(None)),
         }
 
         self.elements = elements
@@ -123,7 +120,6 @@ class FlexTemplate:
                     e["size"] = e["w"]
             for k, t in key_config.items():
                 if k in e and not isinstance(e[k], t):
-                    # pylint: disable=no-member
                     ttype = (
                         t.__name__
                         if isinstance(t, type)
@@ -152,27 +148,61 @@ class FlexTemplate:
             return False
         return None
 
-    def parse_csv(self, infile, delimiter=",", decimal_sep=".", encoding=None):
+    def parse_json(self, infile: os.PathLike, encoding: str = "utf-8"):
+        """
+        Load the template definition from a JSON file.
+        The data must be structured as an array of objects, with names and values exactly
+        equivalent to what would get supplied to load_elements(),
+
+        Arguments:
+
+            infile (string or path-like object): The filepath of the JSON file.
+
+            encoding (string): The character encoding of the file. Default is UTF-8.
+        """
+        with open(infile, encoding=encoding) as f:
+            data = json.load(f)
+            for d in data:
+                fgval = d.get("foreground")
+                if fgval and isinstance(fgval, str):
+                    if fgval.lower().startswith("#"):
+                        d["foreground"] = int(fgval[1:], 16)
+                    else:
+                        raise ValueError(
+                            "If foreground is a string, it must have the form '#rrggbb'."
+                        )
+                bgval = d.get("background")
+                if bgval and isinstance(bgval, str):
+                    if bgval.lower().startswith("#"):
+                        d["background"] = int(bgval[1:], 16)
+                    else:
+                        raise ValueError(
+                            "If background is a string, it must have the form '#rrggbb'."
+                        )
+            self.load_elements(data)
+
+    def parse_csv(
+        self,
+        infile: os.PathLike,
+        delimiter: str = ",",
+        decimal_sep: str = ".",
+        encoding: str = None,
+    ):
         """
         Load the template definition from a CSV file.
 
         Arguments:
 
-            infile (string):
-                The filename of the CSV file.
+            infile (string or path-like object): The filepath of the CSV file.
 
-            delimiter (single character):
-                The character that seperates the fields in the CSV file:
+            delimiter (single character): The character that seperates the fields in the CSV file:
                 Usually a comma, semicolon, or tab.
 
-            decimal_sep (single character):
-                The decimal separator used in the file.
+            decimal_sep (single character): The decimal separator used in the file.
                 Usually either a point or a comma.
 
-            encoding (string):
-                The character encoding of the file.
+            encoding (string): The character encoding of the file.
                 Default is the system default encoding.
-
         """
 
         def _varsep_float(s, default="0"):
@@ -200,6 +230,7 @@ class FlexTemplate:
             ("priority", int, False),
             ("multiline", self._parse_multiline, False),
             ("rotate", _varsep_float, False),
+            ("wrapmode", str, False),
         )
         self.elements = []
         if encoding is None:
@@ -269,11 +300,9 @@ class FlexTemplate:
 
         Arguments:
 
-            text (string):
-                The input text string.
+            text (string): The input text string.
 
-            element_name (string):
-                The name of the template element to fit the text inside.
+            element_name (string): The name of the template element to fit the text inside.
 
         Returns:
             A list of substrings, each of which will fit into the element width
@@ -298,9 +327,11 @@ class FlexTemplate:
         return self.splitting_pdf.multi_cell(
             w=element["x2"] - element["x1"],
             h=element["y2"] - element["y1"],
-            txt=str(text),
+            text=str(text),
             align=element.get("align", ""),
-            split_only=True,
+            dry_run=True,
+            output="LINES",
+            wrapmode=element.get("wrapmode", "WORD"),
         )
 
     def _text(
@@ -321,6 +352,7 @@ class FlexTemplate:
         foreground=0,
         background=None,
         multiline=None,
+        wrapmode="WORD",
         **__,
     ):
         if not text:
@@ -351,16 +383,28 @@ class FlexTemplate:
         pdf.set_xy(x1, y1)
         width, height = x2 - x1, y2 - y1
         if multiline is None:  # write without wrapping/trimming (default)
-            pdf.cell(w=width, h=height, txt=text, border=0, align=align, fill=fill)
+            pdf.cell(w=width, h=height, text=text, border=0, align=align, fill=fill)
         elif multiline:  # automatic word - warp
             pdf.multi_cell(
-                w=width, h=height, txt=text, border=0, align=align, fill=fill
+                w=width,
+                h=height,
+                text=text,
+                border=0,
+                align=align,
+                fill=fill,
+                wrapmode=wrapmode,
             )
         else:  # trim to fit exactly the space defined
             text = pdf.multi_cell(
-                w=width, h=height, txt=text, align=align, split_only=True
+                w=width,
+                h=height,
+                text=text,
+                align=align,
+                wrapmode=wrapmode,
+                dry_run=True,
+                output="LINES",
             )[0]
-            pdf.cell(w=width, h=height, txt=text, border=0, align=align, fill=fill)
+            pdf.cell(w=width, h=height, text=text, border=0, align=align, fill=fill)
 
     def _line(
         self,
@@ -473,9 +517,12 @@ class FlexTemplate:
     ):
         if x is not None or y is not None or w is not None or h is not None:
             warnings.warn(
-                "code39 arguments x/y/w/h are deprecated, please use x1/y1/y2/size instead",
+                (
+                    "code39 arguments x/y/w/h are deprecated since v2.4.4,"
+                    " please use x1/y1/y2/size instead"
+                ),
                 DeprecationWarning,
-                stacklevel=2,
+                stacklevel=get_stack_level(),
             )
         pdf = self.pdf
         if pdf.fill_color.serialize().lower() != _rgb_as_str(foreground):
@@ -533,14 +580,11 @@ class FlexTemplate:
 
         Arguments:
 
-            offsetx, offsety (float):
-                Place the template to move its origin to the given coordinates.
+            offsetx, offsety (float): Place the template to move its origin to the given coordinates.
 
-            rotate (float):
-                Rotate the inserted template around its (offset) origin.
+            rotate (float): Rotate the inserted template around its (offset) origin.
 
-            scale (float):
-                Scale the inserted template by this factor.
+            scale (float): Scale the inserted template by this factor.
         """
         sorted_elements = sorted(self.elements, key=lambda x: x["priority"])
         with self.pdf.local_context():
@@ -585,7 +629,6 @@ class Template(FlexTemplate):
 
     # Disabling this check due to the "format" parameter below:
     # pylint: disable=redefined-builtin
-    # pylint: disable=unused-argument
     def __init__(
         self,
         infile=None,
@@ -602,23 +645,18 @@ class Template(FlexTemplate):
         """
         Arguments:
 
-            infile (str):
-                [**DEPRECATED since 2.2.0**] unused, will be removed in a later version
+            infile (str): [**DEPRECATED since 2.2.0**] unused, will be removed in a later version
 
-            elements (list of dicts):
-                A template definition in a list of dicts.
+            elements (list of dicts): A template definition in a list of dicts.
                 If you omit this, then you need to call either load_elements()
                 or parse_csv() before doing anything else.
 
-            format (str):
-                The page format of the document (eg. "A4" or "letter").
+            format (str): The page format of the document (eg. "A4" or "letter").
 
-            orientation (str):
-                The orientation of the document.
+            orientation (str): The orientation of the document.
                 Possible values are "portrait"/"P" or "landscape"/"L"
 
-            unit (str):
-                The units used in the template definition.
+            unit (str): The units used in the template definition.
                 One of "mm", "cm", "in", "pt", or a number for points per unit.
 
             title (str): The title of the document.
@@ -631,9 +669,9 @@ class Template(FlexTemplate):
         """
         if infile:
             warnings.warn(
-                '"infile" is deprecated, unused and will soon be removed',
+                '"infile" is deprecated since v2.2.0, unused and will soon be removed',
                 DeprecationWarning,
-                stacklevel=2,
+                stacklevel=get_stack_level(),
             )
         for arg in (
             "format",
@@ -645,6 +683,7 @@ class Template(FlexTemplate):
             "creator",
             "keywords",
         ):
+            # nosemgrep: python.lang.security.dangerous-globals-use.dangerous-globals-use
             if not isinstance(locals()[arg], str):
                 raise TypeError(f'Argument "{arg}" must be of type str.')
         pdf = FPDF(format=format, orientation=orientation, unit=unit)
@@ -668,20 +707,18 @@ class Template(FlexTemplate):
 
         Arguments:
 
-            outfile (str):
-                If given, the PDF file will be written to this file name.
+            outfile (str): If given, the PDF file will be written to this file name.
                 Alternatively, the `.pdf.output()` method can be manually called.
 
-            dest (str):
-                [**DEPRECATED since 2.2.0**] unused, will be removed in a later version.
+            dest (str): [**DEPRECATED since 2.2.0**] unused, will be removed in a later version.
         """
         if dest:
             warnings.warn(
-                '"dest" is deprecated, unused and will soon be removed',
+                '"dest" is deprecated since v2.2.0, unused and will soon be removed',
                 DeprecationWarning,
-                stacklevel=2,
+                stacklevel=get_stack_level(),
             )
-        self.pdf.set_font("helvetica", "B", 16)
+        self.pdf.set_font("helvetica", style="B", size=16)
         self.pdf.set_auto_page_break(False, margin=0)
         super().render()
         if outfile:
